@@ -228,6 +228,7 @@ static PlaylistTypes dual_types[] = {
 	PLAYLIST_TYPE2 ("audio/vnd.rn-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
 	PLAYLIST_TYPE2 ("audio/x-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
 	PLAYLIST_TYPE2 ("text/plain", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
+	PLAYLIST_TYPE2 ("application/x-php", NULL, NULL),
 	PLAYLIST_TYPE2 ("audio/x-ms-asx", totem_pl_parser_add_asx, totem_pl_parser_is_asx),
 	PLAYLIST_TYPE2 ("video/x-ms-asf", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
 	PLAYLIST_TYPE2 ("video/x-ms-wmv", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
@@ -1667,8 +1668,7 @@ totem_pl_parser_cleanup_xml (char *contents)
 		/* Broken file? */
 		if (end == NULL)
 			return;
-		if (memmem (needle, end - needle,
-			    "]]>", strlen ("]]>")) != NULL) {
+		if (g_strstr_len (needle, end - needle, "]]>") != NULL) {
 			/* Advance 3 and skip */
 			needle += 3;
 			continue;
@@ -1750,6 +1750,25 @@ totem_pl_parser_ignore_from_mimetype (TotemPlParser *parser, const char *mimetyp
 	return FALSE;
 }
 
+static PlaylistCallback
+totem_pl_parser_get_function_for_mimetype (const char *mimetype)
+{
+	guint i;
+
+	if (mimetype == NULL)
+		return NULL;
+
+	for (i = 0; i < G_N_ELEMENTS(special_types); i++) {
+		if (strcmp (special_types[i].mimetype, mimetype) == 0)
+			return special_types[i].func;
+	}
+	for (i = 0; i < G_N_ELEMENTS(dual_types); i++) {
+		if (strcmp (dual_types[i].mimetype, mimetype) == 0)
+			return dual_types[i].func;
+	}
+	return NULL;
+}
+
 TotemPlParserResult
 totem_pl_parser_parse_internal (TotemPlParser *parser,
 				GFile *file,
@@ -1820,7 +1839,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 	DEBUG(file, g_print ("_get_mime_type_for_name for '%s' returned '%s'\n", uri, mimetype));
 	if (mimetype == NULL || strcmp (UNKNOWN_TYPE, mimetype) == 0
 	    || (g_file_is_native (file) && g_content_type_is_a (mimetype, "text/plain") != FALSE)) {
-	    	char *new_mimetype;
+		char *new_mimetype;
 		new_mimetype = my_g_file_info_get_mime_type_with_data (file, &data, parser);
 		if (new_mimetype) {
 			g_free (mimetype);
@@ -1890,6 +1909,8 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 
 		for (i = 0; i < G_N_ELEMENTS(dual_types) && found == FALSE; i++) {
 			if (strcmp (dual_types[i].mimetype, mimetype) == 0) {
+				PlaylistCallback func;
+
 				DEBUG(file, g_print ("URI '%s' is dual type '%s'\n", uri, mimetype));
 				if (data == NULL) {
 					g_free (mimetype);
@@ -1903,13 +1924,24 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 					mimetype = NULL;
 					break;
 				}
+				/* Now look for the proper function to use */
+				func = totem_pl_parser_get_function_for_mimetype (mimetype);
+				if (func == NULL && mimetype != NULL) {
+					DEBUG(file, g_print ("Ignoring URI '%s' because we couldn't find a playlist parser for '%s'\n", uri, mimetype));
+					ret = TOTEM_PL_PARSER_RESULT_IGNORED;
+					g_free (mimetype);
+					mimetype = NULL;
+					break;
+				} else if (func == NULL) {
+					func = dual_types[i].func;
+				}
 
 				if (base_file == NULL)
 					base_file = g_file_get_parent (file);
 				else
 					base_file = g_object_ref (base_file);
 
-				ret = (* dual_types[i].func) (parser, file, base_file ? base_file : file, parse_data, data);
+				ret = (* func) (parser, file, base_file ? base_file : file, parse_data, data);
 
 				if (base_file != NULL)
 					g_object_unref (base_file);
@@ -1981,11 +2013,11 @@ parse_thread (GSimpleAsyncResult *result, GObject *object, GCancellable *cancell
  * totem_pl_parser_parse_with_base_async:
  * @parser: a #TotemPlParser
  * @uri: the URI of the playlist to parse
- * @base: the base path for relative filenames
+ * @base: (allow-none): the base path for relative filenames, or %NULL
  * @fallback: %TRUE if the parser should add the playlist URI to the
  * end of the playlist on parse failure
- * @cancellable: optional #GCancellable object, or %NULL
- * @callback: a #GAsyncReadyCallback to call when parsing is finished
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @callback: (allow-none): a #GAsyncReadyCallback to call when parsing is finished
  * @user_data: data to pass to the @callback function
  *
  * Starts asynchronous parsing of a playlist given by the absolute URI @uri, using @base to resolve relative paths where appropriate.
@@ -2022,7 +2054,7 @@ totem_pl_parser_parse_with_base_async (TotemPlParser *parser, const char *uri, c
  * totem_pl_parser_parse_with_base:
  * @parser: a #TotemPlParser
  * @uri: the URI of the playlist to parse
- * @base: the base path for relative filenames
+ * @base: (allow-none): the base path for relative filenames, or %NULL
  * @fallback: %TRUE if the parser should add the playlist URI to the
  * end of the playlist on parse failure
  *
@@ -2076,8 +2108,8 @@ totem_pl_parser_parse_with_base (TotemPlParser *parser, const char *uri,
  * @uri: the URI of the playlist to parse
  * @fallback: %TRUE if the parser should add the playlist URI to the
  * end of the playlist on parse failure
- * @cancellable: optional #GCancellable object, or %NULL
- * @callback: a #GAsyncReadyCallback to call when parsing is finished
+ * @cancellable: (allow-none): optional #GCancellable object, or %NULL
+ * @callback: (allow-none): a #GAsyncReadyCallback to call when parsing is finished
  * @user_data: data to pass to the @callback function
  *
  * Starts asynchronous parsing of a playlist given by the absolute URI @uri. @self and @uri are both reffed/copied
@@ -2333,6 +2365,8 @@ totem_pl_parser_mime_type_from_data (gconstpointer data, int len)
 			if (func == dual_types[i].iden)
 				continue;
 			func = dual_types[i].iden;
+			if (func == NULL)
+				continue;
 			res = func (data, len);
 			if (res != NULL) {
 				g_free (mime_type);
